@@ -75,17 +75,18 @@
 #include <limits.h>
 #include <fcntl.h>
 #include <sys/stat.h>
+#include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
+#include <stdint.h>
 
 
 BSON_BEGIN_DECLS
 
 
 #ifdef _MSC_VER
-#include "bson-stdint-win32.h"
 #ifndef __cplusplus
 /* benign redefinition of type */
 #pragma warning(disable : 4142)
@@ -93,7 +94,10 @@ BSON_BEGIN_DECLS
 #define _SSIZE_T_DEFINED
 typedef SSIZE_T ssize_t;
 #endif
+#ifndef _SIZE_T_DEFINED
+#define _SIZE_T_DEFINED
 typedef SIZE_T size_t;
+#endif
 #pragma warning(default : 4142)
 #else
 /*
@@ -107,14 +111,25 @@ typedef SSIZE_T ssize_t;
 #endif
 #pragma warning(default : 4142)
 #endif
+#ifndef PRIi32
 #define PRIi32 "d"
+#endif
+#ifndef PRId32
 #define PRId32 "d"
+#endif
+#ifndef PRIu32
 #define PRIu32 "u"
+#endif
+#ifndef PRIi64
 #define PRIi64 "I64i"
+#endif
+#ifndef PRId64
 #define PRId64 "I64i"
+#endif
+#ifndef PRIu64
 #define PRIu64 "I64u"
+#endif
 #else
-#include "bson-stdint.h"
 #include <inttypes.h>
 #endif
 
