@@ -441,6 +441,40 @@ _dec128_istreq (const char *a, /* IN */
 bool
 bson_decimal128_from_string (const char *string,     /* IN */
                              bson_decimal128_t *dec) /* OUT */
+{
+   return bson_decimal128_from_string_w_len (string, -1, dec);
+}
+
+
+/**
+ *------------------------------------------------------------------------------
+ *
+ * bson_decimal128_from_string_w_len --
+ *
+ *    This function converts @string in the format [+-]ddd[.]ddd[E][+-]dddd to
+ *    decimal128.  Out of range values are converted to +/-Infinity.  Invalid
+ *    strings are converted to NaN. @len is the length of the string, or -1
+ *    meaning the string is null-terminated.
+ *
+ *    If more digits are provided than the available precision allows,
+ *    round to the nearest expressable decimal128 with ties going to even will
+ *    occur.
+ *
+ *    Note: @string must be ASCII only!
+ *
+ * Returns:
+ *    true on success, or false on failure. @dec will be NaN if @str was invalid
+ *    The &bson_decimal128_t converted from @string at @dec.
+ *
+ * Side effects:
+ *    None.
+ *
+ *------------------------------------------------------------------------------
+ */
+bool
+bson_decimal128_from_string_w_len (const char *string,     /* IN */
+                                   int len,                /* IN */
+                                   bson_decimal128_t *dec) /* OUT */
 {
    _bson_uint128_6464_t significand = {0};
 
@@ -495,7 +529,8 @@ bson_decimal128_from_string (const char *string,     /* IN */
    }
 
    /* Read digits */
-   while (isdigit (*str_read) || *str_read == '.') {
+   while (((isdigit (*str_read) || *str_read == '.')) &&
+          (len == -1 || str_read < string + len)) {
       if (*str_read == '.') {
          if (saw_radix) {
             BSON_DECIMAL128_SET_NAN (*dec);
@@ -555,7 +590,7 @@ bson_decimal128_from_string (const char *string,     /* IN */
 #undef SSCANF
    }
 
-   if (*str_read) {
+   if ((len == -1 || str_read < string + len) && *str_read) {
       BSON_DECIMAL128_SET_NAN (*dec);
       return false;
    }
