@@ -33,6 +33,7 @@
 BSON_BEGIN_DECLS
 
 typedef enum {
+   MONGOC_ASYNC_CMD_INITIATE,
    MONGOC_ASYNC_CMD_SETUP,
    MONGOC_ASYNC_CMD_SEND,
    MONGOC_ASYNC_CMD_RECV_LEN,
@@ -47,23 +48,28 @@ typedef struct _mongoc_async_cmd {
    mongoc_async_t *async;
    mongoc_async_cmd_state_t state;
    int events;
+   mongoc_async_cmd_initiate_t initiator;
    mongoc_async_cmd_setup_t setup;
    void *setup_ctx;
    mongoc_async_cmd_cb_t cb;
    void *data;
    bson_error_t error;
-   int64_t start_time;
+   int64_t initiate_delay_ms;
+   int64_t connect_started;
+   int64_t cmd_started;
    int64_t timeout_msec;
    bson_t cmd;
    mongoc_buffer_t buffer;
    mongoc_array_t array;
    mongoc_iovec_t *iovec;
    size_t niovec;
+   size_t bytes_written;
    size_t bytes_to_read;
    mongoc_rpc_t rpc;
    bson_t reply;
    bool reply_needs_cleanup;
    char ns[MONGOC_NAMESPACE_MAX];
+   struct addrinfo *dns_result;
 
    struct _mongoc_async_cmd *next;
    struct _mongoc_async_cmd *prev;
@@ -72,6 +78,10 @@ typedef struct _mongoc_async_cmd {
 mongoc_async_cmd_t *
 mongoc_async_cmd_new (mongoc_async_t *async,
                       mongoc_stream_t *stream,
+                      bool is_setup_done,
+                      struct addrinfo *dns_result,
+                      mongoc_async_cmd_initiate_t initiator,
+                      int64_t initiate_delay_ms,
                       mongoc_async_cmd_setup_t setup,
                       void *setup_ctx,
                       const char *dbname,
