@@ -17,7 +17,6 @@
 
 #include <bson.h>
 
-#include <assert.h>
 #include <fcntl.h>
 #include <stdlib.h>
 #include <string.h>
@@ -46,7 +45,7 @@ typedef struct {
 #pragma pack()
 
 
-BSON_STATIC_ASSERT (sizeof (mongoc_counter_info_t) == 128);
+BSON_STATIC_ASSERT2 (counter_info_t, sizeof (mongoc_counter_info_t) == 128);
 
 
 #pragma pack(1)
@@ -61,17 +60,21 @@ typedef struct {
 #pragma pack()
 
 
-BSON_STATIC_ASSERT (sizeof (mongoc_counters_t) == 64);
+BSON_STATIC_ASSERT2 (counters_t, sizeof (mongoc_counters_t) == 64);
 
+#ifdef MONGOC_ENABLE_SHM_COUNTERS
+/* When counters are enabled at compile time but fail to initiate a shared
+ * memory segment, then fall back to a malloc'd segment. This malloc'd segment
+ * isn't useful to anyone. But by using this fallback, the counter increment
+ * functions can behave the same. I.e. they do not need to have a runtime check
+ * for whether or not initiating the shared memory segment succeeded. */
 static void *gCounterFallback = NULL;
 
-
 #define COUNTER(ident, Category, Name, Description) \
    mongoc_counter_t __mongoc_counter_##ident;
 #include "mongoc-counters.defs"
 #undef COUNTER
 
-
 /**
  * mongoc_counters_use_shm:
  *
@@ -79,14 +82,11 @@ static void *gCounterFallback = NULL;
  *
  * Returns: true if SHM is to be used.
  */
-#if defined(BSON_OS_UNIX) && defined(MONGOC_ENABLE_SHM_COUNTERS)
 static bool
 mongoc_counters_use_shm (void)
 {
    return !getenv ("MONGOC_DISABLE_SHM");
 }
-#endif
-
 
 /**
  * mongoc_counters_calc_size:
@@ -116,6 +116,7 @@ mongoc_counters_calc_size (void)
    return size;
 #endif
 }
+#endif
 
 
 /**
@@ -126,6 +127,7 @@ mongoc_counters_calc_size (void)
 void
 _mongoc_counters_cleanup (void)
 {
+#ifdef MONGOC_ENABLE_SHM_COUNTERS
    if (gCounterFallback) {
       bson_free (gCounterFallback);
       gCounterFallback = NULL;
@@ -139,9 +141,11 @@ _mongoc_counters_cleanup (void)
       shm_unlink (name);
 #endif
    }
+#endif
 }
 
 
+#ifdef MONGOC_ENABLE_SHM_COUNTERS
 /**
  * mongoc_counters_alloc:
  * @size: The size of the shared memory segment.
@@ -167,6 +171,9 @@ mongoc_counters_alloc (size_t size)
    pid = getpid ();
    bson_snprintf (name, sizeof name, "/mongoc-%u", pid);
 
+#ifndef O_NOFOLLOW
+#define O_NOFOLLOW 0
+#endif
    if (-1 == (fd = shm_open (name,
                              O_CREAT | O_EXCL | O_RDWR,
                              S_IRUSR | S_IWUSR | O_NOFOLLOW))) {
@@ -266,7 +273,7 @@ mongoc_counters_register (mongoc_counters_t *counters,
 
    return infos->offset;
 }
-
+#endif
 
 /**
  * mongoc_counters_init:
@@ -277,6 +284,7 @@ mongoc_counters_register (mongoc_counters_t *counters,
 void
 _mongoc_counters_init (void)
 {
+#ifdef MONGOC_ENABLE_SHM_COUNTERS
    mongoc_counter_info_t *info;
    mongoc_counters_t *counters;
    size_t infos_size;
@@ -312,4 +320,5 @@ _mongoc_counters_init (void)
     */
    bson_memory_barrier ();
    counters->size = (uint32_t) size;
+#endif
 }
