@@ -36,14 +36,14 @@ struct _mongoc_read_prefs_t {
 };
 
 
-typedef struct _mongoc_apply_read_prefs_result_t {
-   bson_t *query_with_read_prefs;
+typedef struct _mongoc_assemble_query_result_t {
+   bson_t *assembled_query;
    bool query_owned;
    mongoc_query_flags_t flags;
-} mongoc_apply_read_prefs_result_t;
+} mongoc_assemble_query_result_t;
 
 
-#define READ_PREFS_RESULT_INIT       \
+#define ASSEMBLE_QUERY_RESULT_INIT   \
    {                                 \
       NULL, false, MONGOC_QUERY_NONE \
    }
@@ -52,19 +52,22 @@ const char *
 _mongoc_read_mode_as_str (mongoc_read_mode_t mode);
 
 void
-apply_read_preferences (const mongoc_read_prefs_t *read_prefs,
-                        const mongoc_server_stream_t *server_stream,
-                        const bson_t *query_bson,
-                        mongoc_query_flags_t initial_flags,
-                        mongoc_apply_read_prefs_result_t *result);
+assemble_query (const mongoc_read_prefs_t *read_prefs,
+                const mongoc_server_stream_t *server_stream,
+                const bson_t *query_bson,
+                mongoc_query_flags_t initial_flags,
+                mongoc_assemble_query_result_t *result);
 
 void
-apply_read_prefs_result_cleanup (mongoc_apply_read_prefs_result_t *result);
+assemble_query_result_cleanup (mongoc_assemble_query_result_t *result);
 
 bool
 _mongoc_read_prefs_validate (const mongoc_read_prefs_t *read_prefs,
                              bson_error_t *error);
 
+#define IS_PREF_PRIMARY(_pref) \
+   (!(_pref) || ((_pref)->mode == MONGOC_READ_PRIMARY))
+
 BSON_END_DECLS
 
 
