@@ -113,7 +113,7 @@ typedef union {
 BSON_STATIC_ASSERT2 (sizeof_rpc_header, sizeof (mongoc_rpc_header_t) == 16);
 BSON_STATIC_ASSERT2 (offsetof_rpc_header,
                      offsetof (mongoc_rpc_header_t, opcode) ==
-                     offsetof (mongoc_rpc_reply_t, opcode));
+                        offsetof (mongoc_rpc_reply_t, opcode));
 BSON_STATIC_ASSERT2 (sizeof_reply_header,
                      sizeof (mongoc_rpc_reply_header_t) == 36);
 
@@ -166,6 +166,11 @@ _mongoc_cmd_check_ok (const bson_t *doc,
                       int32_t error_api_version,
                       bson_error_t *error);
 
+bool
+_mongoc_cmd_check_ok_no_wce (const bson_t *doc,
+                             int32_t error_api_version,
+                             bson_error_t *error);
+
 bool
 _mongoc_rpc_decompress (mongoc_rpc_t *rpc_le, uint8_t *buf, size_t buflen);
 
