@@ -24,6 +24,10 @@
 #include "mongoc-host-list.h"
 #include "mongoc-socket-private.h"
 #include "mongoc-trace-private.h"
+#ifdef _WIN32
+#include <Mstcpip.h>
+#include <process.h>
+#endif
 
 #undef MONGOC_LOG_DOMAIN
 #define MONGOC_LOG_DOMAIN "socket"
@@ -33,6 +37,38 @@
    ((expire_at >= 0) && (expire_at < (bson_get_monotonic_time ())))
 
 
+/* either struct sockaddr or void, depending on platform */
+typedef MONGOC_SOCKET_ARG2 mongoc_sockaddr_t;
+
+
+/*
+ *--------------------------------------------------------------------------
+ *
+ * _mongoc_socket_capture_errno --
+ *
+ *       Save the errno state for contextual use.
+ *
+ * Returns:
+ *       None.
+ *
+ * Side effects:
+ *       None.
+ *
+ *--------------------------------------------------------------------------
+ */
+
+static void
+_mongoc_socket_capture_errno (mongoc_socket_t *sock) /* IN */
+{
+#ifdef _WIN32
+   errno = sock->errno_ = WSAGetLastError ();
+#else
+   sock->errno_ = errno;
+#endif
+   TRACE ("setting errno: %d %s", sock->errno_, strerror (sock->errno_));
+}
+
+
 /*
  *--------------------------------------------------------------------------
  *
@@ -93,16 +129,15 @@ _mongoc_socket_setnonblock (int sd)
  */
 
 static bool
-#ifdef _WIN32
-_mongoc_socket_wait (SOCKET sd, /* IN */
-#else
-_mongoc_socket_wait (int sd, /* IN */
-#endif
-                     int events,        /* IN */
-                     int64_t expire_at) /* IN */
+_mongoc_socket_wait (mongoc_socket_t *sock, /* IN */
+                     int events,            /* IN */
+                     int64_t expire_at)     /* IN */
 {
 #ifdef _WIN32
-   WSAPOLLFD pfd;
+   fd_set read_fds;
+   fd_set write_fds;
+   fd_set error_fds;
+   struct timeval timeout_tv;
 #else
    struct pollfd pfd;
 #endif
@@ -112,16 +147,28 @@ _mongoc_socket_wait (int sd, /* IN */
 
    ENTRY;
 
+   BSON_ASSERT (sock);
    BSON_ASSERT (events);
 
-   pfd.fd = sd;
 #ifdef _WIN32
-   pfd.events = events;
-#else
-   pfd.events = events | POLLERR | POLLHUP;
-#endif
-   pfd.revents = 0;
+   FD_ZERO (&read_fds);
+   FD_ZERO (&write_fds);
+   FD_ZERO (&error_fds);
 
+   if (events & POLLIN) {
+      FD_SET (sock->sd, &read_fds);
+   }
+
+   if (events & POLLOUT) {
+      FD_SET (sock->sd, &write_fds);
+   }
+
+   FD_SET (sock->sd, &error_fds);
+#else
+   pfd.fd = sock->sd;
+   pfd.events = events | POLLERR | POLLHUP;
+   pfd.revents = 0;
+#endif
    now = bson_get_monotonic_time ();
 
    for (;;) {
@@ -137,9 +184,20 @@ _mongoc_socket_wait (int sd, /* IN */
       }
 
 #ifdef _WIN32
-      ret = WSAPoll (&pfd, 1, timeout);
+      if (timeout == -1) {
+         /* not WSAPoll: daniel.haxx.se/blog/2012/10/10/wsapoll-is-broken */
+         ret = select (0 /*unused*/, &read_fds, &write_fds, &error_fds, NULL);
+      } else {
+         timeout_tv.tv_sec = timeout / 1000;
+         timeout_tv.tv_usec = (timeout % 1000) * 1000;
+         ret = select (
+            0 /*unused*/, &read_fds, &write_fds, &error_fds, &timeout_tv);
+      }
       if (ret == SOCKET_ERROR) {
-         errno = WSAGetLastError ();
+         _mongoc_socket_capture_errno (sock);
+         ret = -1;
+      } else if (FD_ISSET (sock->sd, &error_fds)) {
+         errno = WSAECONNRESET;
          ret = -1;
       }
 #else
@@ -149,7 +207,8 @@ _mongoc_socket_wait (int sd, /* IN */
       if (ret > 0) {
 /* Something happened, so return that */
 #ifdef _WIN32
-         RETURN (0 != (pfd.revents & (events | POLLHUP | POLLERR)));
+         return (FD_ISSET (sock->sd, &read_fds) ||
+                 FD_ISSET (sock->sd, &write_fds));
 #else
          RETURN (0 != (pfd.revents & events));
 #endif
@@ -161,16 +220,26 @@ _mongoc_socket_wait (int sd, /* IN */
             now = bson_get_monotonic_time ();
 
             if (expire_at < now) {
+               _mongoc_socket_capture_errno (sock);
                RETURN (false);
             } else {
                continue;
             }
          } else {
             /* poll failed for some non-transient reason */
+            _mongoc_socket_capture_errno (sock);
             RETURN (false);
          }
       } else {
-         /* poll timed out */
+         /* ret == 0, poll timed out */
+         if (timeout) {
+            mongoc_counter_streams_timeout_inc ();
+         }
+#ifdef _WIN32
+         sock->errno_ = timeout ? WSAETIMEDOUT : EAGAIN;
+#else
+         sock->errno_ = timeout ? ETIMEDOUT : EAGAIN;
+#endif
          RETURN (false);
       }
    }
@@ -180,20 +249,17 @@ _mongoc_socket_wait (int sd, /* IN */
 /*
  *--------------------------------------------------------------------------
  *
- * _mongoc_socket_poll --
+ * mongoc_socket_poll --
  *
  *       A multi-socket poll helper.
  *
- *       @events: in most cases should be POLLIN or POLLOUT.
- *
  *       @expire_at should be an absolute time at which to expire using
  *       the monotonic clock (bson_get_monotonic_time(), which is in
  *       microseconds). Or zero to not block at all. Or -1 to block
  *       forever.
  *
  * Returns:
- *       true if an event matched. otherwise false.
- *       a timeout will return false.
+ *       The number of sockets ready.
  *
  * Side effects:
  *       None.
@@ -207,7 +273,10 @@ mongoc_socket_poll (mongoc_socket_poll_t *sds, /* IN */
                     int32_t timeout)           /* IN */
 {
 #ifdef _WIN32
-   WSAPOLLFD *pfds;
+   fd_set read_fds;
+   fd_set write_fds;
+   fd_set error_fds;
+   struct timeval timeout_tv;
 #else
    struct pollfd *pfds;
 #endif
@@ -219,41 +288,260 @@ mongoc_socket_poll (mongoc_socket_poll_t *sds, /* IN */
    BSON_ASSERT (sds);
 
 #ifdef _WIN32
-   pfds = (WSAPOLLFD *) bson_malloc (sizeof (*pfds) * nsds);
+   FD_ZERO (&read_fds);
+   FD_ZERO (&write_fds);
+   FD_ZERO (&error_fds);
+
+   for (i = 0; i < nsds; i++) {
+      if (sds[i].events & POLLIN) {
+         FD_SET (sds[i].socket->sd, &read_fds);
+      }
+
+      if (sds[i].events & POLLOUT) {
+         FD_SET (sds[i].socket->sd, &write_fds);
+      }
+
+      FD_SET (sds[i].socket->sd, &error_fds);
+   }
+
+   timeout_tv.tv_sec = timeout / 1000;
+   timeout_tv.tv_usec = (timeout % 1000) * 1000;
+
+   /* not WSAPoll: daniel.haxx.se/blog/2012/10/10/wsapoll-is-broken */
+   ret = select (0 /*unused*/, &read_fds, &write_fds, &error_fds, &timeout_tv);
+   if (ret == SOCKET_ERROR) {
+      errno = WSAGetLastError ();
+      return -1;
+   }
+
+   for (i = 0; i < nsds; i++) {
+      if (FD_ISSET (sds[i].socket->sd, &read_fds)) {
+         sds[i].revents = POLLIN;
+      } else if (FD_ISSET (sds[i].socket->sd, &write_fds)) {
+         sds[i].revents = POLLOUT;
+      } else if (FD_ISSET (sds[i].socket->sd, &error_fds)) {
+         sds[i].revents = POLLHUP;
+      } else {
+         sds[i].revents = 0;
+      }
+   }
 #else
    pfds = (struct pollfd *) bson_malloc (sizeof (*pfds) * nsds);
-#endif
 
    for (i = 0; i < nsds; i++) {
       pfds[i].fd = sds[i].socket->sd;
-#ifdef _WIN32
-      pfds[i].events = sds[i].events;
-#else
       pfds[i].events = sds[i].events | POLLERR | POLLHUP;
-#endif
       pfds[i].revents = 0;
    }
 
-#ifdef _WIN32
-   ret = WSAPoll (pfds, nsds, timeout);
-   if (ret == SOCKET_ERROR) {
-      MONGOC_WARNING ("WSAGetLastError(): %d", WSAGetLastError ());
-      ret = -1;
-   }
-#else
    ret = poll (pfds, nsds, timeout);
-#endif
-
    for (i = 0; i < nsds; i++) {
       sds[i].revents = pfds[i].revents;
    }
 
    bson_free (pfds);
+#endif
 
    return ret;
 }
 
 
+/* https://jira.mongodb.org/browse/CDRIVER-2176 */
+#define MONGODB_KEEPALIVEINTVL 10
+#define MONGODB_KEEPIDLE 300
+#define MONGODB_KEEPALIVECNT 9
+
+#ifdef _WIN32
+static void
+_mongoc_socket_setkeepalive_windows (SOCKET sd)
+{
+   struct tcp_keepalive keepalive;
+   DWORD lpcbBytesReturned = 0;
+   HKEY hKey;
+   DWORD type;
+   DWORD data;
+   DWORD data_size = sizeof data;
+   const char *reg_key =
+      "SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters";
+   keepalive.onoff = true;
+   keepalive.keepalivetime = MONGODB_KEEPIDLE * 1000;
+   keepalive.keepaliveinterval = MONGODB_KEEPALIVEINTVL * 1000;
+   /*
+    * Windows hardcodes probes to 10:
+    * https://msdn.microsoft.com/en-us/library/windows/desktop/dd877220(v=vs.85).aspx
+    * "On Windows Vista and later, the number of keep-alive probes (data
+    * retransmissions) is set to 10 and cannot be changed."
+    *
+    * Note that win2k (and seeminly all versions thereafter) do not set the
+    * registry value by default so there is no way to derive the default value
+    * programmatically. It is however listed in the docs. A user can however
+    * change the default value by setting the registry values.
+    */
+
+   if (RegOpenKeyExA (HKEY_LOCAL_MACHINE, reg_key, 0, KEY_QUERY_VALUE, &hKey) ==
+       ERROR_SUCCESS) {
+      /* https://technet.microsoft.com/en-us/library/cc957549.aspx */
+      DWORD default_keepalivetime = 7200000; /* 2 hours */
+      /* https://technet.microsoft.com/en-us/library/cc957548.aspx */
+      DWORD default_keepaliveinterval = 1000; /* 1 second */
+
+      if (RegQueryValueEx (
+             hKey, "KeepAliveTime", NULL, &type, (LPBYTE) &data, &data_size) ==
+          ERROR_SUCCESS) {
+         if (type == REG_DWORD && data < keepalive.keepalivetime) {
+            keepalive.keepalivetime = data;
+         }
+      } else if (default_keepalivetime < keepalive.keepalivetime) {
+         keepalive.keepalivetime = default_keepalivetime;
+      }
+
+      if (RegQueryValueEx (hKey,
+                           "KeepAliveInterval",
+                           NULL,
+                           &type,
+                           (LPBYTE) &data,
+                           &data_size) == ERROR_SUCCESS) {
+         if (type == REG_DWORD && data < keepalive.keepaliveinterval) {
+            keepalive.keepaliveinterval = data;
+         }
+      } else if (default_keepaliveinterval < keepalive.keepaliveinterval) {
+         keepalive.keepaliveinterval = default_keepaliveinterval;
+      }
+      RegCloseKey (hKey);
+   }
+   if (WSAIoctl (sd,
+                 SIO_KEEPALIVE_VALS,
+                 &keepalive,
+                 sizeof keepalive,
+                 NULL,
+                 0,
+                 &lpcbBytesReturned,
+                 NULL,
+                 NULL) == SOCKET_ERROR) {
+      TRACE ("%s", "Could not set keepalive values");
+   } else {
+      TRACE ("%s", "KeepAlive values updated");
+      TRACE ("KeepAliveTime: %d", keepalive.keepalivetime);
+      TRACE ("KeepAliveInterval: %d", keepalive.keepaliveinterval);
+   }
+}
+#else
+#ifdef MONGOC_TRACE
+static const char *
+_mongoc_socket_sockopt_value_to_name (int value)
+{
+   switch (value) {
+#ifdef TCP_KEEPIDLE
+   case TCP_KEEPIDLE:
+      return "TCP_KEEPIDLE";
+#endif
+#ifdef TCP_KEEPALIVE
+   case TCP_KEEPALIVE:
+      return "TCP_KEEPALIVE";
+#endif
+#ifdef TCP_KEEPINTVL
+   case TCP_KEEPINTVL:
+      return "TCP_KEEPINTVL";
+#endif
+#ifdef TCP_KEEPCNT
+   case TCP_KEEPCNT:
+      return "TCP_KEEPCNT";
+#endif
+   default:
+      MONGOC_WARNING ("Don't know what socketopt %d is", value);
+      return "Unknown option name";
+   }
+}
+#endif
+static void
+_mongoc_socket_set_sockopt_if_less (int sd, int name, int value)
+{
+   int optval = 1;
+   mongoc_socklen_t optlen;
+
+   optlen = sizeof optval;
+   if (getsockopt (sd, IPPROTO_TCP, name, (char *) &optval, &optlen)) {
+      TRACE ("Getting '%s' failed, errno: %d",
+             _mongoc_socket_sockopt_value_to_name (name),
+             errno);
+   } else {
+      TRACE ("'%s' is %d, target value is %d",
+             _mongoc_socket_sockopt_value_to_name (name),
+             optval,
+             value);
+      if (optval > value) {
+         optval = value;
+         if (setsockopt (
+                sd, IPPROTO_TCP, name, (char *) &optval, sizeof optval)) {
+            TRACE ("Setting '%s' failed, errno: %d",
+                   _mongoc_socket_sockopt_value_to_name (name),
+                   errno);
+         } else {
+            TRACE ("'%s' value changed to %d",
+                   _mongoc_socket_sockopt_value_to_name (name),
+                   optval);
+         }
+      }
+   }
+}
+
+static void
+_mongoc_socket_setkeepalive_nix (int sd)
+{
+#if defined(TCP_KEEPIDLE)
+   _mongoc_socket_set_sockopt_if_less (sd, TCP_KEEPIDLE, MONGODB_KEEPIDLE);
+#elif defined(TCP_KEEPALIVE)
+   _mongoc_socket_set_sockopt_if_less (sd, TCP_KEEPALIVE, MONGODB_KEEPIDLE);
+#else
+   TRACE ("%s", "Neither TCP_KEEPIDLE nor TCP_KEEPALIVE available");
+#endif
+
+#ifdef TCP_KEEPINTVL
+   _mongoc_socket_set_sockopt_if_less (
+      sd, TCP_KEEPINTVL, MONGODB_KEEPALIVEINTVL);
+#else
+   TRACE ("%s", "TCP_KEEPINTVL not available");
+#endif
+
+#ifdef TCP_KEEPCNT
+   _mongoc_socket_set_sockopt_if_less (sd, TCP_KEEPCNT, MONGODB_KEEPALIVECNT);
+#else
+   TRACE ("%s", "TCP_KEEPCNT not available");
+#endif
+}
+
+#endif
+static void
+#ifdef _WIN32
+_mongoc_socket_setkeepalive (SOCKET sd) /* IN */
+#else
+_mongoc_socket_setkeepalive (int sd) /* IN */
+#endif
+{
+#ifdef SO_KEEPALIVE
+   int optval = 1;
+
+   ENTRY;
+#ifdef SO_KEEPALIVE
+   if (!setsockopt (
+          sd, SOL_SOCKET, SO_KEEPALIVE, (char *) &optval, sizeof optval)) {
+      TRACE ("%s", "Setting SO_KEEPALIVE");
+#ifdef _WIN32
+      _mongoc_socket_setkeepalive_windows (sd);
+#else
+      _mongoc_socket_setkeepalive_nix (sd);
+#endif
+   } else {
+      TRACE ("%s", "Failed setting SO_KEEPALIVE");
+   }
+#else
+   TRACE ("%s", "SO_KEEPALIVE not available");
+#endif
+   EXIT;
+#endif
+}
+
+
 static bool
 #ifdef _WIN32
 _mongoc_socket_setnodelay (SOCKET sd) /* IN */
@@ -309,34 +597,6 @@ mongoc_socket_errno (mongoc_socket_t *sock) /* IN */
 }
 
 
-/*
- *--------------------------------------------------------------------------
- *
- * _mongoc_socket_capture_errno --
- *
- *       Save the errno state for contextual use.
- *
- * Returns:
- *       None.
- *
- * Side effects:
- *       None.
- *
- *--------------------------------------------------------------------------
- */
-
-static void
-_mongoc_socket_capture_errno (mongoc_socket_t *sock) /* IN */
-{
-#ifdef _WIN32
-   errno = sock->errno_ = WSAGetLastError ();
-#else
-   sock->errno_ = errno;
-#endif
-   TRACE ("setting errno: %d %s", sock->errno_, strerror (sock->errno_));
-}
-
-
 /*
  *--------------------------------------------------------------------------
  *
@@ -411,7 +671,7 @@ mongoc_socket_accept_ex (mongoc_socket_t *sock, /* IN */
                          uint16_t *port)        /* OUT */
 {
    mongoc_socket_t *client;
-   struct sockaddr_in addr = {0};
+   struct sockaddr_storage addr = {0};
    mongoc_socklen_t addrlen = sizeof addr;
    bool try_again = false;
    bool failed = false;
@@ -427,7 +687,7 @@ mongoc_socket_accept_ex (mongoc_socket_t *sock, /* IN */
 
 again:
    errno = 0;
-   sd = accept (sock->sd, (struct sockaddr *) &addr, &addrlen);
+   sd = accept (sock->sd, (mongoc_sockaddr_t *) &addr, &addrlen);
 
    _mongoc_socket_capture_errno (sock);
 #ifdef _WIN32
@@ -438,7 +698,7 @@ again:
    try_again = (failed && _mongoc_socket_errno_is_again (sock));
 
    if (failed && try_again) {
-      if (_mongoc_socket_wait (sock->sd, POLLIN, expire_at)) {
+      if (_mongoc_socket_wait (sock, POLLIN, expire_at)) {
          GOTO (again);
       }
       RETURN (NULL);
@@ -457,7 +717,13 @@ again:
    client->sd = sd;
 
    if (port) {
-      *port = ntohs (addr.sin_port);
+      if (addr.ss_family == AF_INET) {
+         struct sockaddr_in *tmp = (struct sockaddr_in *) &addr;
+         *port = ntohs (tmp->sin_port);
+      } else {
+         struct sockaddr_in6 *tmp = (struct sockaddr_in6 *) &addr;
+         *port = ntohs (tmp->sin6_port);
+      }
    }
 
    if (!_mongoc_socket_setnodelay (client->sd)) {
@@ -505,36 +771,23 @@ mongoc_socket_bind (mongoc_socket_t *sock,       /* IN */
 }
 
 
-/*
- *--------------------------------------------------------------------------
- *
- * mongoc_socket_close --
- *
- *       Closes the underlying socket.
- *
- *       In general, you probably don't want to handle the result from
- *       this. That could cause race conditions in the case of preemtion
- *       during system call (EINTR).
- *
- * Returns:
- *       0 on success, -1 on failure.
- *
- * Side effects:
- *       None.
- *
- *--------------------------------------------------------------------------
- */
-
 int
 mongoc_socket_close (mongoc_socket_t *sock) /* IN */
 {
+   bool owned;
+
    ENTRY;
 
    BSON_ASSERT (sock);
 
 #ifdef _WIN32
+   owned = (sock->pid == (int) _getpid ());
+
    if (sock->sd != INVALID_SOCKET) {
-      shutdown (sock->sd, SD_BOTH);
+      if (owned) {
+         shutdown (sock->sd, SD_BOTH);
+      }
+
       if (0 == closesocket (sock->sd)) {
          sock->sd = INVALID_SOCKET;
       } else {
@@ -544,8 +797,13 @@ mongoc_socket_close (mongoc_socket_t *sock) /* IN */
    }
    RETURN (0);
 #else
+   owned = (sock->pid == (int) getpid ());
+
    if (sock->sd != -1) {
-      shutdown (sock->sd, SHUT_RDWR);
+      if (owned) {
+         shutdown (sock->sd, SHUT_RDWR);
+      }
+
       if (0 == close (sock->sd)) {
          sock->sd = -1;
       } else {
@@ -585,7 +843,8 @@ mongoc_socket_connect (mongoc_socket_t *sock,       /* IN */
    bool failed = false;
    int ret;
    int optval;
-   mongoc_socklen_t optlen = sizeof optval;
+   /* getsockopt parameter types vary, we check in CheckCompiler.m4 */
+   mongoc_socklen_t optlen = (mongoc_socklen_t) sizeof optval;
 
    ENTRY;
 
@@ -607,7 +866,7 @@ mongoc_socket_connect (mongoc_socket_t *sock,       /* IN */
    }
 
    if (failed && try_again) {
-      if (_mongoc_socket_wait (sock->sd, POLLOUT, expire_at)) {
+      if (_mongoc_socket_wait (sock, POLLOUT, expire_at)) {
          optval = -1;
          ret = getsockopt (
             sock->sd, SOL_SOCKET, SO_ERROR, (char *) &optval, &optlen);
@@ -698,9 +957,9 @@ mongoc_socket_listen (mongoc_socket_t *sock, /* IN */
  *
  * mongoc_socket_new --
  *
- *       Create a new socket.
+ *       Create a new socket and store the current process id on it.
  *
- *       Free the result mongoc_socket_destroy().
+ *       Free the result with mongoc_socket_destroy().
  *
  * Returns:
  *       A newly allocated socket.
@@ -740,13 +999,21 @@ mongoc_socket_new (int domain,   /* IN */
       GOTO (fail);
    }
 
-   if (domain != AF_UNIX && !_mongoc_socket_setnodelay (sd)) {
-      MONGOC_WARNING ("Failed to enable TCP_NODELAY.");
+   if (domain != AF_UNIX) {
+      if (!_mongoc_socket_setnodelay (sd)) {
+         MONGOC_WARNING ("Failed to enable TCP_NODELAY.");
+      }
+      _mongoc_socket_setkeepalive (sd);
    }
 
    sock = (mongoc_socket_t *) bson_malloc0 (sizeof *sock);
    sock->sd = sd;
    sock->domain = domain;
+#ifdef _WIN32
+   sock->pid = (int) _getpid ();
+#else
+   sock->pid = (int) getpid ();
+#endif
 
    RETURN (sock);
 
@@ -812,7 +1079,7 @@ again:
    if (failed) {
       _mongoc_socket_capture_errno (sock);
       if (_mongoc_socket_errno_is_again (sock) &&
-          _mongoc_socket_wait (sock->sd, POLLIN, expire_at)) {
+          _mongoc_socket_wait (sock, POLLIN, expire_at)) {
          GOTO (again);
       }
    }
@@ -1156,13 +1423,16 @@ mongoc_socket_sendv (mongoc_socket_t *sock,  /* IN */
          BSON_ASSERT (iovcnt - cur);
          BSON_ASSERT (iov[cur].iov_len);
       } else if (OPERATION_EXPIRED (expire_at)) {
+         if (expire_at > 0) {
+            mongoc_counter_streams_timeout_inc ();
+         }
          GOTO (CLEANUP);
       }
 
       /*
        * Block on poll() until our desired condition is met.
        */
-      if (!_mongoc_socket_wait (sock->sd, POLLOUT, expire_at)) {
+      if (!_mongoc_socket_wait (sock, POLLOUT, expire_at)) {
          GOTO (CLEANUP);
       }
    }
@@ -1196,8 +1466,9 @@ mongoc_socket_getsockname (mongoc_socket_t *sock,     /* IN */
 char *
 mongoc_socket_getnameinfo (mongoc_socket_t *sock) /* IN */
 {
-   struct sockaddr addr;
-   mongoc_socklen_t len = sizeof addr;
+   /* getpeername parameter types vary, we check in CheckCompiler.m4 */
+   struct sockaddr_storage addr;
+   mongoc_socklen_t len = (mongoc_socklen_t) sizeof addr;
    char *ret;
    char host[BSON_HOST_NAME_MAX + 1];
 
@@ -1205,13 +1476,17 @@ mongoc_socket_getnameinfo (mongoc_socket_t *sock) /* IN */
 
    BSON_ASSERT (sock);
 
-   if ((0 == getpeername (sock->sd, &addr, &len)) &&
-       (0 == getnameinfo (&addr, len, host, sizeof host, NULL, 0, 0))) {
-      ret = bson_strdup (host);
-      RETURN (ret);
+   if (getpeername (sock->sd, (struct sockaddr *) &addr, &len)) {
+      RETURN (NULL);
    }
 
-   RETURN (NULL);
+   if (getnameinfo (
+          (struct sockaddr *) &addr, len, host, sizeof host, NULL, 0, 0)) {
+      RETURN (NULL);
+   }
+
+   ret = bson_strdup (host);
+   RETURN (ret);
 }
 
 
@@ -1222,7 +1497,7 @@ mongoc_socket_check_closed (mongoc_socket_t *sock) /* IN */
    char buf[1];
    ssize_t r;
 
-   if (_mongoc_socket_wait (sock->sd, POLLIN, 0)) {
+   if (_mongoc_socket_wait (sock, POLLIN, 0)) {
       sock->errno_ = 0;
 
       r = recv (sock->sd, buf, 1, MSG_PEEK);
