@@ -103,7 +103,9 @@ mongoc_stream_destroy (mongoc_stream_t *stream)
 {
    ENTRY;
 
-   BSON_ASSERT (stream);
+   if (!stream) {
+      EXIT;
+   }
 
    BSON_ASSERT (stream->destroy);
 
@@ -312,9 +314,8 @@ mongoc_stream_get_base_stream (mongoc_stream_t *stream) /* IN */
 }
 
 
-static mongoc_stream_t *
+mongoc_stream_t *
 mongoc_stream_get_root_stream (mongoc_stream_t *stream)
-
 {
    BSON_ASSERT (stream);
 
@@ -383,86 +384,6 @@ CLEANUP:
    return rval;
 }
 
-
-/*
- *--------------------------------------------------------------------------
- *
- * mongoc_stream_wait --
- *
- *       Internal helper, poll a single stream until it connects.
- *
- *       For now, only the POLLOUT (connected) event is supported.
- *
- *       @expire_at should be an absolute time at which to expire using
- *       the monotonic clock (bson_get_monotonic_time(), which is in
- *       microseconds). expire_at of 0 or -1 is prohibited.
- *
- * Returns:
- *       true if an event matched. otherwise false.
- *       a timeout will return false.
- *
- * Side effects:
- *       None.
- *
- *--------------------------------------------------------------------------
- */
-
-bool
-mongoc_stream_wait (mongoc_stream_t *stream, int64_t expire_at)
-{
-   mongoc_stream_poll_t poller;
-   int64_t now;
-   int32_t timeout_msec;
-   ssize_t ret;
-
-   ENTRY;
-
-   BSON_ASSERT (stream);
-   BSON_ASSERT (expire_at > 0);
-
-   poller.stream = stream;
-   poller.events = POLLOUT;
-   poller.revents = 0;
-
-   now = bson_get_monotonic_time ();
-
-   for (;;) {
-      /* TODO CDRIVER-804 use int64_t for timeouts consistently */
-      timeout_msec = (int32_t) BSON_MIN ((expire_at - now) / 1000L, INT32_MAX);
-      if (timeout_msec < 0) {
-         timeout_msec = 0;
-      }
-
-      ret = mongoc_stream_poll (&poller, 1, timeout_msec);
-
-      if (ret > 0) {
-         /* an event happened, return true if POLLOUT else false */
-         RETURN (0 != (poller.revents & POLLOUT));
-      } else if (ret < 0) {
-         /* poll itself failed */
-
-         TRACE ("errno is: %d", errno);
-         if (MONGOC_ERRNO_IS_AGAIN (errno)) {
-            now = bson_get_monotonic_time ();
-
-            if (expire_at < now) {
-               RETURN (false);
-            } else {
-               continue;
-            }
-         } else {
-            /* poll failed for some non-transient reason */
-            RETURN (false);
-         }
-      } else {
-         /* poll timed out */
-         RETURN (false);
-      }
-   }
-
-   return true;
-}
-
 bool
 mongoc_stream_check_closed (mongoc_stream_t *stream)
 {
@@ -479,6 +400,17 @@ mongoc_stream_check_closed (mongoc_stream_t *stream)
    RETURN (ret);
 }
 
+bool
+mongoc_stream_timed_out (mongoc_stream_t *stream)
+{
+   ENTRY;
+
+   BSON_ASSERT (stream);
+
+   /* for e.g. a file stream there is no timed_out function */
+   RETURN (stream->timed_out && stream->timed_out (stream));
+}
+
 bool
 _mongoc_stream_writev_full (mongoc_stream_t *stream,
                             mongoc_iovec_t *iov,
