@@ -39,25 +39,16 @@ struct _mongoc_write_concern_t {
    char *wtag;
    bool frozen;
    bson_t compiled;
-   bson_t compiled_gle;
    bool is_default;
 };
 
 
 mongoc_write_concern_t *
-_mongoc_write_concern_new_from_iter (bson_iter_t *iter);
-bool
-_mongoc_write_concern_iter_is_valid (bson_iter_t *iter);
-const bson_t *
-_mongoc_write_concern_get_gle (mongoc_write_concern_t *write_concern);
+_mongoc_write_concern_new_from_iter (const bson_iter_t *iter,
+                                     bson_error_t *error);
 const bson_t *
 _mongoc_write_concern_get_bson (mongoc_write_concern_t *write_concern);
 bool
-_mongoc_write_concern_is_default (const mongoc_write_concern_t *write_concern);
-bool
-_mongoc_write_concern_validate (const mongoc_write_concern_t *write_concern,
-                                bson_error_t *error);
-bool
 _mongoc_parse_wc_err (const bson_t *doc, bson_error_t *error);
 
 BSON_END_DECLS
